/* 

File:    amg.c
Purpose: Create an Audio Manager (AUDIO_TS.IFO)

dvda-author  - Author a DVD-Audio DVD

(C) Dave Chapman <dave@dchapman.com> 2005

The latest version can be found at http://dvd-audio.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "audio.h"

int create_amg(char* audiotsdir,fileinfo_t* files, int* ntracks, int ngroups, int* atsi_sectors) {
  int i,j;
  FILE* fpout;
  char outfile[560];
  uint8_t amg[2048*3];
  int numtitles[9];
  int ntitletracks[9][99];
  uint64_t titlelength[9][99];
  int titleset;
  int totaltitles=0;
  int sectoroffset;
  int titleintitleset;

  j=0;
  titleset=0;
  numtitles[titleset]=0;
  ntitletracks[titleset][numtitles[titleset]]=0;

  while ((j < ntracks[titleset]) && (titleset < ngroups)) {
    //fprintf(stderr,"HERE ngroups=%d, titleset=%d, j=%d, ntracks[titleset]=%d\n",ngroups,titleset,j,ntracks[titleset]);

    ntitletracks[titleset][numtitles[titleset]]=1;
    titlelength[titleset][numtitles[titleset]]=0;
    titlelength[titleset][numtitles[titleset]]+=files[titleset*99+j].PTS_length;
    j++;
    while ((j < ntracks[titleset]) && (files[titleset*99+j].samplerate==files[titleset*99+j-1].samplerate) && (files[titleset*99+j].bitspersample==files[titleset*99+j-1].bitspersample) && (files[titleset*99+j].channels==files[titleset*99+j-1].channels)) {
      ntitletracks[titleset][numtitles[titleset]]++;
      titlelength[titleset][numtitles[titleset]]+=files[titleset*99+j].PTS_length;
      j++;
    }
    totaltitles++;
    numtitles[titleset]++;
    if (j==ntracks[titleset]) { 
      j=0; titleset++; numtitles[titleset]=0;
    }
  }

  memset(amg,0,sizeof(amg));
  memcpy(&amg[0],"DVDAUDIO-AMG",12);
  uint32_copy(&amg[12],5); // Last sector in AMG
  uint32_copy(&amg[28],2); // Last sector in AMGI
  uint16_copy(&amg[32],0x0012); // DVD Specifications
  uint16_copy(&amg[38],0x0001); // Number of Volumes
  uint16_copy(&amg[40],0x0001); // Current Volume
  amg[42]=1;  // Disc Side
  amg[47]=1;  // ??? Maybe something to do with SV ??
  uint32_copy(&amg[48],0);    // Sector Pointer to AUDIO_SV.IFO
  amg[62]=0;  // Number of video titlesets
  amg[63]=ngroups;  // Number of audio titlesets
  uint32_copy(&amg[128],0x07ff);

  uint32_copy(&amg[0xc4],1);  // Pointer to sector 2
  uint32_copy(&amg[0xc8],2);  // Pointer to sector 3

  /* Sector 2 */

  i=0x800;
  uint16_copy(&amg[i],totaltitles);
  i+=2;
  uint16_copy(&amg[i],4+14*totaltitles-1);
  i+=2;

  sectoroffset=6;  // Size of AUDIO_TS.IFO+AUDIO_TS.BUP
  titleset=0;
  titleintitleset=0;
  for (j=0;j<totaltitles;j++) {
    amg[i]=0x80|(titleset+1); // ??? what is this?
    i++;
    amg[i]=ntitletracks[titleset][titleintitleset];
    i++;
    i+=2;
    uint32_copy(&amg[i],titlelength[titleset][titleintitleset]);
    i+=4;
    amg[i]=titleset+1;  // Titleset number
    i++;
    amg[i]=titleintitleset+1;
    i++;
    uint32_copy(&amg[i],sectoroffset); // Pointer to ATSI
    i+=4;
    titleintitleset++;
    if (titleintitleset==numtitles[titleset]) {
      sectoroffset+=(files[titleset*99+ntracks[titleset]-1].last_sector+1)+atsi_sectors[titleset]*2;
      titleintitleset=0;
      titleset++;
    }
  }

  /* Sector 3 */

  memcpy(&amg[4096],&amg[2048],2048);

  snprintf(outfile,sizeof(outfile),"%s/AUDIO_TS.IFO",audiotsdir);
  fpout=fopen(outfile,"wb+");
  fwrite(amg,1,sizeof(amg),fpout);
  fclose(fpout);

  snprintf(outfile,sizeof(outfile),"%s/AUDIO_TS.BUP",audiotsdir);
  fpout=fopen(outfile,"wb+");
  fwrite(amg,1,sizeof(amg),fpout);
  fclose(fpout);

  return(0);
}
